﻿using AbstractSoftwareInstallationContracts.BusinessLogicsContracts;
using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using Microsoft.Extensions.Logging;
using AbstractSoftwareInstallationDataModels;
using AbstractSoftwareInstallationBusinessLogic.MailWorker;

namespace AbstractSoftwareInstallationBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IClientLogic _clientLogic;
        private readonly AbstractMailWorker _mailWorker;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage OrderStorage, IClientLogic clientLogic, AbstractMailWorker mailWorker)
        {
            _logger = logger;
            _orderStorage = OrderStorage;
            _clientLogic = clientLogic;
            _mailWorker = mailWorker;
        }
        public void SendOrderStatusMail(int clientId, string subject, string text)
        {
            var client = _clientLogic.ReadElement(new ClientSearchModel
            {
                Id = clientId
            });
            Console.WriteLine($"{clientId}, {subject}, {text}, {client}, {client?.Email}");
            if (client == null)
            {
                throw new ArgumentNullException($"Клиент с Id:{clientId} не найден.");
            }
            _mailWorker.MailSendAsync(new MailSendInfoBindingModel
            {
                MailAddress = client.Email,
                Subject = subject,
                Text = text
            });
        }
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.PackageId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор документа", nameof(model.PackageId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество пакетов в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderID:{Id}.Sum:{ Sum}. PackageId: { PackageId}", model.Id, model.Sum, model.PackageId);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Orderid:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }
            model.Status = OrderStatus.Принят;
            var result = _orderStorage.Insert(model);
            if (result == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            SendOrderStatusMail(result.ClientId, $"Новый заказ создан. Номер заказа #{result.Id}", $"Заказ #{result.Id} от {result.DateCreate} на сумму {result.Sum:0.00} принят");
            return true;
        }
        public bool StatusUpdate(OrderBindingModel rawModel, OrderStatus _newStatus)
        {
            var viewModel = _orderStorage.GetElement(new OrderSearchModel
            {
                Id = rawModel.Id
            });

            if (viewModel == null)
            {
                _logger.LogWarning("Order model not found");
                return false;
            }
            OrderBindingModel model = new OrderBindingModel
            {
                Id = viewModel.Id,
                PackageId = viewModel.PackageId,
                Status = viewModel.Status,
                DateCreate = viewModel.DateCreate,
                DateImplement = viewModel.DateImplement,
                Count = viewModel.Count,
                Sum = viewModel.Sum,
                ImplementerId = viewModel.ImplementerId
            };
            if (rawModel.ImplementerId.HasValue)
            {
                model.ImplementerId = rawModel.ImplementerId;
            }

            CheckModel(model, false);
            _logger.LogInformation("STAT {model.Status}, {_newStatus}", model.Status, _newStatus);

            if (model.Status + 1 != _newStatus)
            {
                _logger.LogWarning("Status update to " + _newStatus.ToString() + " operation failed. Order status incorrect.");
                return false;
            }
            model.Status = _newStatus;
            if (model.Status == OrderStatus.Выдан) model.DateImplement = DateTime.Now;
            var result = _orderStorage.Update(model);
            if (result == null)
            {
                model.Status--;
                _logger.LogInformation("Update operation failed");
                return false;
            }
            SendOrderStatusMail(result.ClientId, $"Изменен статус заказа #{result.Id}", $"Заказ #{result.Id} изменен статус на {result.Status}");
            return true;
        }
        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выполняется);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выдан);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Готов);
        }
    }
}
